package ch.zhaw.projekt2.turnierverwaltung;

import javafx.application.Platform;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class TournamentDecorator implements IsObservable{
    private Tournament tournament;
    private FileIO fileIO;
    private List<IsObserver> listener = new ArrayList<>();
    private ExecutorService executorService;
    private FactoryDecorator factoryDecorator;

    public TournamentDecorator(FileIO fileIO){
        setFileIO(fileIO);
        addListener(new IsObserver() {
            @Override
            public void update() {
                if(tournament != null){

                    saveTournament();
                }
            }
        });
        executorService = Executors.newFixedThreadPool(1);
    }

    public void setFactoryDecorator(FactoryDecorator factoryDecorator) {
        this.factoryDecorator = factoryDecorator;
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setTournament(Tournament tournament) {
        this.tournament = tournament;
    }

    public Tournament getTournament() {
        return tournament;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }


    public void saveTournament(){
        executorService.execute(new saveTask());
        factoryDecorator.clearMessage(false);
        String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("save");
        factoryDecorator.printMessageToFooter(msg + new Date().toString(),false);
    }



    public void createTournament(String name, Tournament.Type type){

        try {
            if(fileIO.tournamentExists(name)){
                //TODO:Logger
                String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("tournamentExists");
                factoryDecorator.printMessageToFooter(msg, true);
                return;
            }
            Tournament tournament = new Tournament(name, type);
            fileIO.saveTournament(tournament);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace();
            //TODO: Logger
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidName");
            factoryDecorator.printMessageToFooter(msg, true);
        } catch (Tournament.InvalidTypeException e) {
            e.printStackTrace();
            //TODO: Logger
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidMode");
            factoryDecorator.printMessageToFooter(msg, true);

        } catch (IOException e) {
            e.printStackTrace();
            //TODO: Logger
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("IOException");
            factoryDecorator.printMessageToFooter(msg, true);
        }

    }

    public void deleteTournament(FileIO.TournamentFile tournamentFile){
        try {
            fileIO.deleteTournament(tournamentFile);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (IOException e) {
            e.printStackTrace();  //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("IOException");
            factoryDecorator.printMessageToFooter(msg, true);
        }
    }

    public void createNewGameSchedule() {
        //TODO: logging
        try {
            tournament.createGameSchedule();
            factoryDecorator.clearMessage(true);
        } catch (Tournament.NumberOfParticipantInvalidException e) {
            e.printStackTrace();
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("numberParticipant");
            factoryDecorator.printMessageToFooter(msg, true);
        }
        informListener();
    }

    public void savePlayer(String firstName, String name, String phoneNumber, String dateOfBirth){
        try {
            tournament.saveParticipant(new Player(firstName, name, phoneNumber, dateOfBirth));
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidName");
            factoryDecorator.printMessageToFooter(msg, true);
        } catch (Person.InvalidPhoneNumberException e) {
            e.printStackTrace(); //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidPhone");
            factoryDecorator.printMessageToFooter(msg, true);
        } catch (Player.InvalidDateException e) {
            e.printStackTrace();
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidDate");
            factoryDecorator.printMessageToFooter(msg, true);
        }
    }

    public void deleteParticipant(Participant participant){
        try {
            tournament.removeParticipant(participant);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (Tournament.ParticipantNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("participantNotExist");
            factoryDecorator.printMessageToFooter(msg, true);
        }
    }

    public void savePlace(String name){
        try {
            tournament.savePlace(new Place(name));
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("invalidName");
            factoryDecorator.printMessageToFooter(msg, true);
        }
    }

    public void deletePlace(Place place){
        try {
            tournament.removePlace(place);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (Tournament.PlaceNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
            String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("placeNotExist");
            factoryDecorator.printMessageToFooter(msg, true);
        }
    }


    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }

    public void closeApplication() throws InterruptedException {
        executorService.shutdown();
        executorService.awaitTermination(2000, TimeUnit.MILLISECONDS);
    }


    private class saveTask implements Runnable {

        @Override
        public void run() {
            try {
                fileIO.saveTournament(tournament);
            } catch (IOException e) {
                e.printStackTrace();
                Platform.runLater(() -> {
                    String msg = factoryDecorator.getLanguageConfigurator().getSelectedLanguage("IOException");
                    factoryDecorator.printMessageToFooter(msg, true);
                });
            }
        }
    }

}
